#include <stdio.h>
#include <time.h>
#include <stdlib.h>

#include "AsciiAPI.h"

#ifdef WIN32
#include <windows.h>
#include <direct.h>
#include "strptime.h"
#define SIGNATURE __declspec(dllexport)
#define SNPRINTF _snprintf
#else
#define SIGNATURE
#define SNPRINTF snprintf
#endif

Utils *g_pUtils = NULL;
char *g_pszInputFilename = NULL;
char *g_pszWorkingDirectory = NULL;
static char g_exceptionBuffer[8192];

#include "SimpleMOCinput.logic.h"


SIGNATURE void ModuleMain(RCHANDLE treeRoot, RCHANDLE dummy, RCINT64 dummier)
{
    static char tmpBuf[105];
    FILE *fp = NULL;
    unsigned currentLineNo = 0;

#define CONTROLLED_RUN(x)                                                                           \
        {                                                                                           \
            int result = x;                                                                         \
            if (0 != result) {                                                                      \
                SNPRINTF(g_exceptionBuffer, sizeof(g_exceptionBuffer), "Failed to execute:\n%s\n(result:%d)\n", #x, result); \
                g_pUtils->pfnThrowException(g_exceptionBuffer);                                     \
                return;                                                                             \
            }                                                                                       \
        }

    fp = fopen(g_pszInputFilename, "r");
    if (!fp) {
        SNPRINTF(g_exceptionBuffer, sizeof(g_exceptionBuffer), "Couldn't open '%s' for reading!\n", g_pszInputFilename);
        g_pUtils->pfnThrowException(g_exceptionBuffer);
    }

    if (chdir(g_pszWorkingDirectory)) {
	SNPRINTF(g_exceptionBuffer, sizeof(g_exceptionBuffer), "Failed to change dir to \"%s\"!\n", g_pszWorkingDirectory);
	g_pUtils->pfnThrowException(g_exceptionBuffer);
    }

    while(!feof(fp)) {
        int foundRecord = 0;
        currentLineNo++;

        memset(tmpBuf, 0, sizeof(tmpBuf));
        if (!fgets(tmpBuf, sizeof(tmpBuf), fp))
	    break;
        if (strlen(tmpBuf)<23)
            break;

	if (!foundRecord) {
	    char prefix[3];
	    strncpy(prefix, &tmpBuf[0], 2);
	    prefix[2] = 0;
	    if (!strcmp(prefix, "10")) {
		HEADER r_header;
		foundRecord = 1;
		{
		    char *garbage = NULL;
		    char temp[6];
		    strncpy(temp, &tmpBuf[2], 5);
		    temp[5] = 0;
		    { size_t x=strlen(temp)-1; while (x>=0 && (temp[x]==' '||temp[x]=='\t'||temp[x]=='\r'||temp[x]=='\n')) temp[x--]=0; }
		    strncpy(r_header.SENDER, temp, 6);
		}
		{
		    char *garbage = NULL;
		    char temp[6];
		    strncpy(temp, &tmpBuf[7], 5);
		    temp[5] = 0;
		    { size_t x=strlen(temp)-1; while (x>=0 && (temp[x]==' '||temp[x]=='\t'||temp[x]=='\r'||temp[x]=='\n')) temp[x--]=0; }
		    strncpy(r_header.RECIPIENT, temp, 6);
		}
		{
		    char *garbage = NULL;
		    char temp[6];
		    strncpy(temp, &tmpBuf[12], 5);
		    temp[5] = 0;
		    { size_t x=strlen(temp)-1; while (x>=0 && (temp[x]==' '||temp[x]=='\t'||temp[x]=='\r'||temp[x]=='\n')) temp[x--]=0; }
		    strncpy(r_header.FSN, temp, 6);
		}
		{
		    char *garbage = NULL;
		    char temp[7];
		    strncpy(temp, &tmpBuf[17], 6);
		    temp[6] = 0;
		    {
			struct tm FILE_CREATION_DATE_tm;
			memset(&FILE_CREATION_DATE_tm, 0, sizeof(struct tm));
			FILE_CREATION_DATE_tm.tm_isdst = -1;
			if (!strptime(temp, "%y%m%d", &FILE_CREATION_DATE_tm)) {
			    SNPRINTF(g_exceptionBuffer, sizeof(g_exceptionBuffer), "Cannot convert to time from '%s' (line %d)\n", temp, currentLineNo);
			    g_pUtils->pfnThrowException(g_exceptionBuffer);
			}
			r_header.FILE_CREATION_DATE = FILE_CREATION_DATE_tm;
		    }
		}
		ProcessHEADER(treeRoot, &r_header);
	    }
	}
	if (!foundRecord) {
	    char prefix[3];
	    strncpy(prefix, &tmpBuf[0], 2);
	    prefix[2] = 0;
	    if (!strcmp(prefix, "20")) {
		MOC r_moc;
		foundRecord = 1;
		{
		    char *garbage = NULL;
		    char temp[16];
		    strncpy(temp, &tmpBuf[2], 15);
		    temp[15] = 0;
		    { size_t x=strlen(temp)-1; while (x>=0 && (temp[x]==' '||temp[x]=='\t'||temp[x]=='\r'||temp[x]=='\n')) temp[x--]=0; }
		    strncpy(r_moc.IMSI, temp, 16);
		}
		{
		    char *garbage = NULL;
		    char temp[22];
		    strncpy(temp, &tmpBuf[17], 21);
		    temp[21] = 0;
		    { size_t x=strlen(temp)-1; while (x>=0 && (temp[x]==' '||temp[x]=='\t'||temp[x]=='\r'||temp[x]=='\n')) temp[x--]=0; }
		    strncpy(r_moc.CALLED_NUMBER, temp, 22);
		}
		{
		    char *garbage = NULL;
		    char temp[2];
		    strncpy(temp, &tmpBuf[38], 1);
		    temp[1] = 0;
		    r_moc.NATIONAL_IND = strtol(temp, &garbage, 10);
		}
		{
		    char *garbage = NULL;
		    char temp[15];
		    strncpy(temp, &tmpBuf[39], 14);
		    temp[14] = 0;
		    {
			struct tm CALL_DATE_TIME_tm;
			memset(&CALL_DATE_TIME_tm, 0, sizeof(struct tm));
			CALL_DATE_TIME_tm.tm_isdst = -1;
			if (!strptime(temp, "%Y%m%d%H%M%S", &CALL_DATE_TIME_tm)) {
			    SNPRINTF(g_exceptionBuffer, sizeof(g_exceptionBuffer), "Cannot convert to time from '%s' (line %d)\n", temp, currentLineNo);
			    g_pUtils->pfnThrowException(g_exceptionBuffer);
			}
			r_moc.CALL_DATE_TIME = CALL_DATE_TIME_tm;
		    }
		}
		{
		    char *garbage = NULL;
		    char temp[8];
		    strncpy(temp, &tmpBuf[53], 7);
		    temp[7] = 0;
		    r_moc.DURATION = strtol(temp, &garbage, 10);
		}
		{
		    char *garbage = NULL;
		    char temp[16];
		    strncpy(temp, &tmpBuf[60], 15);
		    temp[15] = 0;
		    { size_t x=strlen(temp)-1; while (x>=0 && (temp[x]==' '||temp[x]=='\t'||temp[x]=='\r'||temp[x]=='\n')) temp[x--]=0; }
		    strncpy(r_moc.MSCID, temp, 16);
		}
		{
		    char *garbage = NULL;
		    char temp[6];
		    strncpy(temp, &tmpBuf[75], 5);
		    temp[5] = 0;
		    { size_t x=strlen(temp)-1; while (x>=0 && (temp[x]==' '||temp[x]=='\t'||temp[x]=='\r'||temp[x]=='\n')) temp[x--]=0; }
		    strncpy(r_moc.CALLEDCOUNTRYCODE, temp, 6);
		}
		{
		    char *garbage = NULL;
		    char temp[10];
		    strncpy(temp, &tmpBuf[80], 9);
		    temp[9] = 0;
		    r_moc.CHARGE = ((double)strtol(temp, &garbage, 10))/1000.0;
		}
		{
		    char *garbage = NULL;
		    char temp[7];
		    strncpy(temp, &tmpBuf[89], 6);
		    temp[6] = 0;
		    r_moc.CHARGEABLE_UNITS = strtol(temp, &garbage, 10);
		}
		{
		    char *garbage = NULL;
		    char temp[9];
		    strncpy(temp, &tmpBuf[95], 8);
		    temp[8] = 0;
		    r_moc.EXCHANGE_RATE = ((double)strtol(temp, &garbage, 10))/100000.0;
		}
		ProcessMOC(treeRoot, &r_moc);
	    }
	}
	if (!foundRecord) {
	    SNPRINTF(g_exceptionBuffer, sizeof(g_exceptionBuffer), "Line %d: unrecognized line type\n%s\n", currentLineNo, tmpBuf);
	    g_pUtils->pfnThrowException(g_exceptionBuffer);
	}
    }

    fclose(fp);
}

SIGNATURE void ModuleShutdown(RCHANDLE treeRoot, RCHANDLE dummy, RCINT64 idxdummy)
{
    free(g_pszInputFilename);
}

SIGNATURE void ModuleVersion(RCHANDLE treeRoot, RCHANDLE dummy, RCINT64 idxdummy)
{
    *(int *)dummy = (int) GetMinorVersionProduced();
}

Callback myfunctions[] = {
    {"RC-Version", ModuleVersion},
    {"RC-Main", ModuleMain},
    {"RC-Shutdown", ModuleShutdown},
    {NULL,NULL}
};

SIGNATURE Callback *getAndSetCallbacks(Utils *pUtils, char *pszInputASCIIfilename, char *workingDirectory)
{
    g_pUtils = pUtils;
    g_pszInputFilename = strdup(pszInputASCIIfilename);
    g_pszWorkingDirectory = strdup(workingDirectory);
    return myfunctions;
}
